﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon.Entities
{
    public class OaDepartIncome
    {
        public string Year { get; set; }
        public string Depart { get; set; }
        public string DepartId { get; set; }
        public decimal AmountTotal { get; set; }
        public decimal AmountTarget { get; set; }
        public int QuantityUnSplit { get; set; }
        public decimal AmountUnSplit { get; set; }
        public string Rate { get; set; }
        public DateTime OperateTime { get; set; }
        public List<OaDepartIncomeDetail> Details { get; set; }
    }

    public class OaDepartIncomeDetail
    {
        public string Type { get; set; }
        public string TypeEx { get; set; }
        public decimal AmountClaim { get; set; }
        public decimal AmountIn { get; set; }
        public decimal AmountOut { get; set; }
        public decimal AmountTotal { get; set; }
    }

    public class RequestOaContract
    {
        /// <summary>
        /// 表单模版code（与应用绑定中保持一致）
        /// </summary>
        public string templateCode { get; set; } = Configs.templateCodeContract;

        /// <summary>
        /// 导出数据的起始时间.eg:2020-06-30
        /// </summary>
        public string beginDateTime { get; set; }

        /// <summary>
        /// 导出数据的截止创建时间.eg:2020-06-30
        /// </summary>
        public string endDateTime { get; set; }

        /// <summary>
        /// 权限id
        /// </summary>
        public string rightId { get; set; } = Configs.templateCodeContractRightId;

        /// <summary>
        /// 数据id
        /// </summary>
        public string dataId { get; set; }
    }
    public class OaContract
    {
        public string success { get; set; }
        public string code { get; set; }
        public string message { get; set; }
        public OaContractEntity data { get; set; }

    }
    public class OaContractEntity
    {
        public Definition definition { get; set; }
        public class Definition
        {
            public Form form { get; set; }
            public class Form
            {
                public string formId { get; set; }
                public string formName { get; set; }
            }

            public List<Field> fields { get; set; }

            public class Field
            {
                public string name { get; set; }
                public string display { get; set; }
                public string fieldType { get; set; }
                public string inputType { get; set; }
                public string formatType { get; set; }
                public string fieldLength { get; set; }
                public string digitNum { get; set; }
                public string enumId { get; set; }
                public string isNotNull { get; set; }
                public string auth { get; set; }
            }
        }

        public List<OaContractEntityData> data { get; set; }
    }
    public class OaContractEntityData   
    {
        public MasterData masterData { get; set; }
        public SubData subData { get; set; }

        public class MasterData
        {
            /// <summary>
            /// 单据编号
            /// </summary>
            [StringLength(100)]
            public FormValue Field0063 { get; set; }

            /// <summary>
            /// 流水号
            /// </summary>
            [StringLength(100)]
            public FormValue Field0081 { get; set; }

            /// <summary>
            /// 年份
            /// </summary>
            [StringLength(100)]
            public FormValue Field0080 { get; set; }

            /// <summary>
            /// 流水号后缀
            /// </summary>
            [StringLength(100)]
            public FormValue Field0082 { get; set; }

            /// <summary>
            /// 名称单位
            /// </summary>
            [StringLength(20)]
            public FormValue Field0060 { get; set; }

            /// <summary>
            /// 协议编号
            /// </summary>
            [StringLength(100)]
            public FormValue Field0062 { get; set; }

            /// <summary>
            /// 申请人
            /// </summary>
            [StringLength(20)]
            public FormValue Field0059 { get; set; }

            /// <summary>
            /// 申请时间
            /// </summary>
            public FormValue Field0043 { get; set; }

            /// <summary>
            /// 申请人部门
            /// </summary>
            [StringLength(20)]
            public FormValue Field0078 { get; set; }

            /// <summary>
            /// 部门编码
            /// </summary>
            [StringLength(255)]
            public FormValue Field0079 { get; set; }

            /// <summary>
            /// 申请人一级部门
            /// </summary>
            [StringLength(20)]
            public FormValue Field0096 { get; set; }

            /// <summary>
            /// 申请人一级部门编码
            /// </summary>
            [StringLength(255)]
            public FormValue Field0097 { get; set; }

            /// <summary>
            /// 合同所属部门
            /// </summary>
            [StringLength(20)]
            public FormValue Field0084 { get; set; }

            /// <summary>
            /// 合同所属部门编码
            /// </summary>
            [StringLength(255)]
            public FormValue Field0083 { get; set; }

            /// <summary>
            /// 合作项目
            /// </summary>
            [StringLength(100)]
            public FormValue Field0001 { get; set; }

            /// <summary>
            /// 申请单位
            /// </summary>
            [StringLength(20)]
            public FormValue Field0058 { get; set; }

            /// <summary>
            /// 承检承接部门
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0002 { get; set; }

            /// <summary>
            /// 项目负责人
            /// </summary>
            [StringLength(20)]
            public FormValue Field0003 { get; set; }

            /// <summary>
            /// 合作单位名称
            /// </summary>
            [StringLength(100)]
            public FormValue Field0004 { get; set; }

            /// <summary>
            /// 合作单位地址
            /// </summary>
            [StringLength(100)]
            public FormValue Field0005 { get; set; }

            /// <summary>
            /// 合作单位法人
            /// </summary>
            [StringLength(100)]
            public FormValue Field0006 { get; set; }

            /// <summary>
            /// 合作单位企业信用代码
            /// </summary>
            [StringLength(18)]
            public FormValue Field0007 { get; set; }

            /// <summary>
            /// 合作单位联系人职务
            /// </summary>
            [StringLength(100)]
            public FormValue Field0008 { get; set; }

            /// <summary>
            /// 合作单位联系电话
            /// </summary>
            [StringLength(100)]
            public FormValue Field0009 { get; set; }

            /// <summary>
            /// 付款单位
            /// </summary>
            [StringLength(100)]
            public FormValue Field0048 { get; set; }

            /// <summary>
            /// 协议类型
            /// </summary>
            [StringLength(20)]
            public FormValue Field0011 { get; set; }

            /// <summary>
            /// 长期合同
            /// </summary>
            [StringLength(20)]
            public FormValue Field0072 { get; set; }

            /// <summary>
            /// 协议年份
            /// </summary>
            [StringLength(100)]
            public FormValue Field0012 { get; set; }

            /// <summary>
            /// 合作开始日期
            /// </summary>
            public FormValue Field0045 { get; set; }

            /// <summary>
            /// 合作结束日期
            /// </summary>
            public FormValue Field0046 { get; set; }

            /// <summary>
            /// 享受折扣输入
            /// </summary>
            [StringLength(20)]
            public FormValue Field0014 { get; set; }

            /// <summary>
            /// 享受折扣
            /// </summary>
            [StringLength(5)]
            public FormValue Field0019 { get; set; }

            /// <summary>
            /// 合作类型
            /// </summary>
            [StringLength(20)]
            public FormValue Field0015 { get; set; }

            /// <summary>
            /// 其他说明
            /// </summary>
            [StringLength(100)]
            public FormValue Field0017 { get; set; }

            /// <summary>
            /// 合同总金额
            /// </summary>
            [StringLength(20)]
            public FormValue Field0020 { get; set; }

            /// <summary>
            /// 结算方式
            /// </summary>
            [StringLength(20)]
            public FormValue Field0022 { get; set; }

            /// <summary>
            /// 首次付款金额
            /// </summary>
            [StringLength(20)]
            public FormValue Field0023 { get; set; }

            /// <summary>
            /// 合同份数
            /// </summary>
            [StringLength(20)]
            public FormValue Field0033 { get; set; }

            /// <summary>
            /// 合同附件
            /// </summary>
            [StringLength(20)]
            public FormValue Field0013 { get; set; }

            /// <summary>
            /// 合同归档-扫描件
            /// </summary>
            [StringLength(20)]
            public FormValue Field0071 { get; set; }

            /// <summary>
            /// LIMS协议编号
            /// </summary>
            [StringLength(100)]
            public FormValue Field0085 { get; set; }

            /// <summary>
            /// 承检承接部门意见
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0025 { get; set; }

            /// <summary>
            /// 加签法务
            /// </summary>
            [StringLength(20)]
            public FormValue Field0056 { get; set; }

            /// <summary>
            /// 业务发展部意见
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0026 { get; set; }

            /// <summary>
            /// 财务管理部意见
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0027 { get; set; }

            /// <summary>
            /// 事业部分管院长意见
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0028 { get; set; }

            /// <summary>
            /// 业务分管院长意见
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0029 { get; set; }

            /// <summary>
            /// 法务意见
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0030 { get; set; }

            /// <summary>
            /// 院长意见
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0031 { get; set; }

            /// <summary>
            /// 折扣说明公式
            /// </summary>
            [StringLength(100)]
            public FormValue Field0064 { get; set; }

            /// <summary>
            /// 首次付款时间
            /// </summary>
            public FormValue Field0070 { get; set; }

            /// <summary>
            /// 合同所属单位
            /// </summary>
            [StringLength(20)]
            public FormValue Field0074 { get; set; }

            /// <summary>
            /// 法人意见
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0075 { get; set; }

            /// <summary>
            /// 判断是否上传扫描件
            /// </summary>
            [StringLength(20)]
            public FormValue Field0088 { get; set; }

            /// <summary>
            /// 盖章时间
            /// </summary>
            public FormValue Field0089 { get; set; }

            /// <summary>
            /// 合作单位企业信用代码当前位数
            /// </summary>
            [StringLength(100)]
            public FormValue Field0091 { get; set; }

            /// <summary>
            /// 合同状态
            /// </summary>
            [StringLength(20)]
            public FormValue Field0092 { get; set; }

            /// <summary>
            /// 作废时间
            /// </summary>
            public FormValue Field0094 { get; set; }

            /// <summary>
            /// 合同作废原因
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0095 { get; set; }

            /// <summary>
            /// 加签院领导
            /// </summary>
            [StringLength(4000)]
            public FormValue Field0098 { get; set; }

            /// <summary>
            /// 电子签-印章名称
            /// </summary>
            [StringLength(1000)]
            public FormValue Field0099 { get; set; }

            /// <summary>
            /// 电子签-是否电子签
            /// </summary>
            [StringLength(20)]
            public FormValue Field0100 { get; set; }

            /// <summary>
            /// 电子签-落章位置
            /// </summary>
            [StringLength(100)]
            public FormValue Field0101 { get; set; }

            /// <summary>
            /// 电子签-实际用章时间
            /// </summary>
            public FormValue Field0102 { get; set; }

            /// <summary>
            /// 电子签-是否盖章
            /// </summary>
            [StringLength(20)]
            public FormValue Field0103 { get; set; }

            /// <summary>
            /// 电子签-是否需要领导审批
            /// </summary>
            [StringLength(20)]
            public FormValue Field0104 { get; set; }

            /// <summary>
            /// 电子签-印章管理员
            /// </summary>
            [StringLength(20)]
            public FormValue Field0105 { get; set; }

            /// <summary>
            /// 电子签-印章编号
            /// </summary>
            [StringLength(100)]
            public FormValue Field0106 { get; set; }

        }

        public class SubData
        {

            public List<Formson_0765> formson_0765 { get; set; }
            public class Formson_0765
            {
                /// <summary>
                /// 印章名称
                /// </summary>
                [StringLength(1000)]
                public FormValue Field0065 { get; set; }

                /// <summary>
                /// 用章时间
                /// </summary>
                public FormValue Field0067 { get; set; }

                /// <summary>
                /// 是否盖章
                /// </summary>
                [StringLength(20)]
                public FormValue Field0066 { get; set; }

                /// <summary>
                /// 是否电子章
                /// </summary>
                [StringLength(20)]
                public FormValue Field0068 { get; set; }

                /// <summary>
                /// 印章管理员
                /// </summary>
                [StringLength(20)]
                public FormValue Field0073 { get; set; }

                /// <summary>
                /// 是否需要领导审批
                /// </summary>
                [StringLength(20)]
                public FormValue Field0077 { get; set; }

                /// <summary>
                /// 落章位置
                /// </summary>
                [StringLength(100)]
                public FormValue Field0090 { get; set; }

                /// <summary>
                /// 印章编号
                /// </summary>
                [StringLength(100)]
                public FormValue Field0093 { get; set; }
            }
        }
    }

    public class FormValue
    {
        public string value { get; set; }
        public string showValue { get; set; }
    }


    public class Contract
    {
        /// <summary>
        /// 单据编号
        /// Field0063
        /// </summary>
        [StringLength(100)]
        public string ContractNumber { get; set; }

        /// <summary>
        /// 流水号
        /// Field0081
        /// </summary>
        [StringLength(100)]
        public string SerialNumber { get; set; }

        /// <summary>
        /// 年份
        /// Field0080
        /// </summary>
        [StringLength(100)]
        public string Year { get; set; }

        /// <summary>
        /// 流水号后缀
        /// </summary>
        [StringLength(100)]
        public string Field0082 { get; set; }

        /// <summary>
        /// 名称单位
        /// </summary>
        [StringLength(20)]
        public string Field0060 { get; set; }

        /// <summary>
        /// 协议编号
        /// </summary>
        [StringLength(100)]
        public string Field0062 { get; set; }

        /// <summary>
        /// 申请人
        /// Field0059
        /// </summary>
        [StringLength(20)]
        public string ApplierName { get; set; }

        /// <summary>
        /// 申请时间
        /// Field0043
        /// </summary>
        public string ApplyTime { get; set; }

        /// <summary>
        /// 申请人部门
        /// Field0078
        /// </summary>
        [StringLength(20)]
        public string ApplyDepart { get; set; }

        /// <summary>
        /// 部门编码
        /// Field0079
        /// </summary>
        [StringLength(255)]
        public string DepartCode { get; set; }

        /// <summary>
        /// 申请人一级部门
        /// Field0096
        /// </summary>
        [StringLength(20)]
        public string DepartmentName { get; set; }

        /// <summary>
        /// 申请人一级部门编码
        /// Field0097
        /// </summary>
        [StringLength(255)]
        public string DepartmentCode { get; set; }

        /// <summary>
        /// 合同所属部门
        /// </summary>
        [StringLength(20)]
        public string Field0084 { get; set; }

        /// <summary>
        /// 合同所属部门编码
        /// </summary>
        [StringLength(255)]
        public string Field0083 { get; set; }

        /// <summary>
        /// 合作项目
        /// Field0001
        /// </summary>
        [StringLength(100)]
        public string Name { get; set; }

        /// <summary>
        /// 申请单位
        /// </summary>
        [StringLength(20)]
        public string Field0058 { get; set; }

        /// <summary>
        /// 承检承接部门
        /// Field0002
        /// </summary>
        [StringLength(4000)]
        public string DepartmentTest { get; set; }

        /// <summary>
        /// 项目负责人
        /// Field0003
        /// </summary>
        [StringLength(20)]
        public string LeaderName { get; set; }

        /// <summary>
        /// 合作单位名称
        /// Field0004
        /// </summary>
        [StringLength(100)]
        public string CustomerName { get; set; }

        /// <summary>
        /// 合作单位地址
        /// Field0005
        /// </summary>
        [StringLength(100)]
        public string CustomerAddress { get; set; }

        /// <summary>
        /// 合作单位法人
        /// Field0006
        /// </summary>
        [StringLength(100)]
        public string CustomerLeaderName { get; set; }

        /// <summary>
        /// 合作单位企业信用代码
        /// Field0007
        /// </summary>
        [StringLength(18)]
        public string CustomerTax { get; set; }

        /// <summary>
        /// 合作单位联系人职务
        /// Field0008
        /// </summary>
        [StringLength(100)]
        public string CustomerContactName { get; set; }

        /// <summary>
        /// 合作单位联系电话
        /// Field0009
        /// </summary>
        [StringLength(100)]
        public string CustomerContactNumber { get; set; }

        /// <summary>
        /// 付款单位
        /// Field0048
        /// </summary>
        [StringLength(100)]
        public string PayerName { get; set; }

        /// <summary>
        /// 协议类型
        /// Field0011
        /// </summary>
        [StringLength(20)]
        public string Type { get; set; }

        /// <summary>
        /// 长期合同
        /// </summary>
        [StringLength(20)]
        public string Field0072 { get; set; }

        /// <summary>
        /// 协议年份
        /// Field0012
        /// </summary>
        [StringLength(100)]
        public string AgreeYear { get; set; }

        /// <summary>
        /// 合作开始日期
        /// Field0045
        /// </summary>
        public string AgreeBeginTime { get; set; }

        /// <summary>
        /// 合作结束日期
        /// Field0046
        /// </summary>
        public string AgreeEndTime { get; set; }

        /// <summary>
        /// 享受折扣输入
        /// Field0014
        /// </summary>
        [StringLength(20)]
        public string RateText { get; set; }

        /// <summary>
        /// 享受折扣
        /// Field0019
        /// </summary>
        [StringLength(5)]
        public string Rate { get; set; }

        /// <summary>
        /// 合作类型
        /// Field0015
        /// </summary>
        [StringLength(20)]
        public string AgreeType { get; set; }

        /// <summary>
        /// 其他说明
        /// Field0017
        /// </summary>
        [StringLength(100)]
        public string Remark { get; set; }

        /// <summary>
        /// 合同总金额
        /// Field0020
        /// </summary>
        [StringLength(20)]
        public string Amount { get; set; }

        /// <summary>
        /// 结算方式
        /// Field0022
        /// </summary>
        [StringLength(20)]
        public string PayType { get; set; }

        /// <summary>
        /// 首次付款金额
        /// Field0023
        /// </summary>
        [StringLength(20)]
        public string AmountFirstPay { get; set; }

        /// <summary>
        /// 合同份数
        /// Field0033
        /// </summary>
        [StringLength(20)]
        public string Quantity { get; set; }

        /// <summary>
        /// 合同附件
        /// </summary>
        [StringLength(20)]
        public string Field0013 { get; set; }

        /// <summary>
        /// 合同归档-扫描件
        /// </summary>
        [StringLength(20)]
        public string Field0071 { get; set; }

        /// <summary>
        /// LIMS协议编号
        /// Field0085
        /// </summary>
        [StringLength(100)]
        public string LimsNumber { get; set; }

        /// <summary>
        /// 承检承接部门意见
        /// </summary>
        [StringLength(4000)]
        public string Field0025 { get; set; }

        /// <summary>
        /// 加签法务
        /// </summary>
        [StringLength(20)]
        public string Field0056 { get; set; }

        /// <summary>
        /// 业务发展部意见
        /// </summary>
        [StringLength(4000)]
        public string Field0026 { get; set; }

        /// <summary>
        /// 财务管理部意见
        /// </summary>
        [StringLength(4000)]
        public string Field0027 { get; set; }

        /// <summary>
        /// 事业部分管院长意见
        /// </summary>
        [StringLength(4000)]
        public string Field0028 { get; set; }

        /// <summary>
        /// 业务分管院长意见
        /// </summary>
        [StringLength(4000)]
        public string Field0029 { get; set; }

        /// <summary>
        /// 法务意见
        /// </summary>
        [StringLength(4000)]
        public string Field0030 { get; set; }

        /// <summary>
        /// 院长意见
        /// </summary>
        [StringLength(4000)]
        public string Field0031 { get; set; }

        /// <summary>
        /// 折扣说明公式
        /// </summary>
        [StringLength(100)]
        public string Field0064 { get; set; }

        /// <summary>
        /// 首次付款时间
        /// Field0070
        /// </summary>
        public string FirstPayTime { get; set; }

        /// <summary>
        /// 合同所属单位
        /// </summary>
        [StringLength(20)]
        public string Field0074 { get; set; }

        /// <summary>
        /// 法人意见
        /// </summary>
        [StringLength(4000)]
        public string Field0075 { get; set; }

        /// <summary>
        /// 判断是否上传扫描件
        /// </summary>
        [StringLength(20)]
        public string Field0088 { get; set; }

        /// <summary>
        /// 盖章时间
        /// Field0089
        /// </summary>
        public string StampTime { get; set; }

        /// <summary>
        /// 合作单位企业信用代码当前位数
        /// </summary>
        [StringLength(100)]
        public string Field0091 { get; set; }

        /// <summary>
        /// 合同状态
        /// </summary>
        [StringLength(20)]
        public string Field0092 { get; set; }

        /// <summary>
        /// 作废时间
        /// </summary>
        public string Field0094 { get; set; }

        /// <summary>
        /// 合同作废原因
        /// </summary>
        [StringLength(4000)]
        public string Field0095 { get; set; }

        /// <summary>
        /// 加签院领导
        /// </summary>
        [StringLength(4000)]
        public string Field0098 { get; set; }

        /// <summary>
        /// 电子签-印章名称
        /// </summary>
        [StringLength(1000)]
        public string Field0099 { get; set; }

        /// <summary>
        /// 电子签-是否电子签
        /// </summary>
        [StringLength(20)]
        public string Field0100 { get; set; }

        /// <summary>
        /// 电子签-落章位置
        /// </summary>
        [StringLength(100)]
        public string Field0101 { get; set; }

        /// <summary>
        /// 电子签-实际用章时间
        /// </summary>
        public string Field0102 { get; set; }

        /// <summary>
        /// 电子签-是否盖章
        /// </summary>
        [StringLength(20)]
        public string Field0103 { get; set; }

        /// <summary>
        /// 电子签-是否需要领导审批
        /// </summary>
        [StringLength(20)]
        public string Field0104 { get; set; }

        /// <summary>
        /// 电子签-印章管理员
        /// </summary>
        [StringLength(20)]
        public string Field0105 { get; set; }

        /// <summary>
        /// 电子签-印章编号
        /// </summary>
        [StringLength(100)]
        public string Field0106 { get; set; }

        ///// <summary>
        ///// 印章名称
        ///// </summary>
        //[StringLength(1000)]
        //public string Field0065 { get; set; }

        ///// <summary>
        ///// 用章时间
        ///// </summary>
        //public string Field0067 { get; set; }

        ///// <summary>
        ///// 是否盖章
        ///// </summary>
        //[StringLength(20)]
        //public string Field0066 { get; set; }

        ///// <summary>
        ///// 是否电子章
        ///// </summary>
        //[StringLength(20)]
        //public string Field0068 { get; set; }

        ///// <summary>
        ///// 印章管理员
        ///// </summary>
        //[StringLength(20)]
        //public string Field0073 { get; set; }

        ///// <summary>
        ///// 是否需要领导审批
        ///// </summary>
        //[StringLength(20)]
        //public string Field0077 { get; set; }

        ///// <summary>
        ///// 落章位置
        ///// </summary>
        //[StringLength(100)]
        //public string Field0090 { get; set; }

        ///// <summary>
        ///// 印章编号
        ///// </summary>
        //[StringLength(100)]
        //public string Field0093 { get; set; }
    }
}
